# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"Config": "00_core.ipynb",
         "config": "00_core.ipynb",
         "download_url": "00_core.ipynb",
         "pd.options.display.max_rows": "02_chars.ipynb",
         "pd.options.display.max_columns": "02_chars.ipynb",
         "list_datasets": "01_datasets.ipynb",
         "datasetsdf": "01_datasets.ipynb",
         "list_dataset_files": "01_datasets.ipynb",
         "read_download_info": "01_datasets.ipynb",
         "download_all_datasets": "01_datasets.ipynb",
         "download_dataset_file": "01_datasets.ipynb",
         "downloaddf": "01_datasets.ipynb",
         "read_dataset_file": "01_datasets.ipynb",
         "download_urls_file": "01_datasets.ipynb",
         "read_urls_file": "01_datasets.ipynb",
         "urlsdf": "01_datasets.ipynb",
         "get_rows_from_datasetdf": "01_datasets.ipynb",
         "show_first_rows": "01_datasets.ipynb",
         "get_textblocks_from_dataset": "01_datasets.ipynb",
         "show_first_textblocks": "01_datasets.ipynb",
         "get_text_from_rowindex": "01_datasets.ipynb",
         "get_url_from_rowindex": "01_datasets.ipynb",
         "find_textblocks_with_chars": "01_datasets.ipynb",
         "getContextAroundWord": "01_datasets.ipynb",
         "chardatadir": "02_chars.ipynb",
         "charname": "02_chars.ipynb",
         "charcategory": "02_chars.ipynb",
         "charsubcategory": "02_chars.ipynb",
         "charblock": "02_chars.ipynb",
         "blockfamily": "02_chars.ipynb",
         "unicodeblocks": "02_chars.ipynb",
         "unicodecategories": "02_chars.ipynb",
         "unicodefamilies": "02_chars.ipynb",
         "NormChange": "02_chars.ipynb",
         "NormResult": "02_chars.ipynb",
         "TextNormalizer": "02_chars.ipynb"}

modules = ["core.py",
           "datasets.py",
           "chars.py"]

doc_url = "https://laurentprudhon.github.io/frenchtext/"

git_url = "https://github.com/laurentprudhon/frenchtext/tree/master/"

def custom_doc_links(name): return None
